@@ -60,7 +60,18 @@ enum OggzFlags {
    * \a flags = OGGZ_READ | OGGZ_AUTO will allow seeking on Speex,
    * Vorbis, FLAC, Theora, CMML and all Annodex streams in units of
    * milliseconds, once all bos pages have been delivered. */
-  OGGZ_AUTO         = 0x20
+  OGGZ_AUTO         = 0x20,
+
+  /**
+   * Prefix
+   */
+  OGGZ_PREFIX       = 0x40,
+
+  /**
+   * Suffix
+   */
+  OGGZ_SUFFIX       = 0x80
+
 };
 
 enum OggzStopCtl {
@@ -123,12 +134,14 @@ enum OggzError {
   OGGZ_ERR_NOSEEK                       = -13,
 
   /** Reading was stopped by an OggzReadCallback returning OGGZ_STOP_OK
-   * before any input bytes were consumed */
-  OGGZ_ERR_READ_STOP_OK                 = -14,
+   * or writing was stopped by an  OggzWriteHungry callback returning
+   * OGGZ_STOP_OK */
+  OGGZ_ERR_STOP_OK                      = -14,
 
   /** Reading was stopped by an OggzReadCallback returning OGGZ_STOP_ERR
-   * before any input bytes were consumed */
-  OGGZ_ERR_READ_STOP_ERR                = -15,
+   * or writing was stopped by an OggzWriteHungry callback returning
+   * OGGZ_STOP_ERR */
+  OGGZ_ERR_STOP_ERR                     = -15,
 
   /** no data available from IO, try again */
   OGGZ_ERR_IO_AGAIN                     = -16,
