@@ -198,6 +198,9 @@ struct _OGGZ {
   off_t offset; /* offset of current page start */
   off_t offset_data_begin; /* offset of unit 0 page start */
 
+  long run_blocksize; /* blocksize to use for oggz_run() */
+  int cb_next;
+
   OggzVector * streams;
   int all_at_eos; /* all streams are at eos */
 
@@ -221,6 +224,8 @@ OGGZ * oggz_write_init (OGGZ * oggz);
 int oggz_write_flush (OGGZ * oggz);
 OGGZ * oggz_write_close (OGGZ * oggz);
 
+int oggz_map_return_value_to_error (int cb_ret);
+
 int oggz_get_bos (OGGZ * oggz, long serialno);
 ogg_int64_t oggz_get_unit (OGGZ * oggz, long serialno, ogg_int64_t granulepos);
 
