@@ -59,6 +59,8 @@
 /* Define to 0 or 1 */
 #define ALWAYS_FLUSH 0
 
+#define OGGZ_WRITE_EMPTY (-707)
+
 /* #define ZPACKET_CMP */
 
 #ifdef ZPACKET_CMP
@@ -201,7 +203,7 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
   ogg_packet * new_op;
   unsigned char * new_buf;
   int b_o_s, e_o_s, bos_auto;
-  int strict;
+  int strict, prefix, suffix;
 
 #ifdef DEBUG
   printf ("oggz_write_feed: IN\n");
@@ -221,8 +223,10 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
   printf ("oggz_write_feed: (%010ld) FLUSH: %d\n", serialno, flush);
 #endif
 
-  /* Cache strict */
+  /* Cache strict, prefix, suffix */
   strict = !(oggz->flags & OGGZ_NONSTRICT);
+  prefix = (oggz->flags & OGGZ_PREFIX);
+  suffix = (oggz->flags & OGGZ_SUFFIX);
 
   /* Set bos, eos to canonical values */
   bos_auto = (op->b_o_s == -1) ? 1 : 0;
@@ -237,7 +241,7 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
 	return OGGZ_ERR_BOS;
     }
 
-    if (b_o_s || !strict) {
+    if (b_o_s || !strict || suffix) {
       stream = oggz_add_stream (oggz, serialno);
     } else {
       return OGGZ_ERR_BAD_SERIALNO;
@@ -245,23 +249,24 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
   } else {
     if (bos_auto) b_o_s = 0;
 
-    if (strict && stream->e_o_s)
+    if (!suffix && strict && stream->e_o_s)
       return OGGZ_ERR_EOS;
   }
 
   /* Check packet against mapping restrictions */
   if (strict) {
     if (op->bytes < 0) return OGGZ_ERR_BAD_BYTES;
-    if (b_o_s != stream->b_o_s) return OGGZ_ERR_BAD_B_O_S;
+    if (!suffix && b_o_s != stream->b_o_s) return OGGZ_ERR_BAD_B_O_S;
 
     if (op->granulepos != -1 && op->granulepos < stream->granulepos)
       return OGGZ_ERR_BAD_GRANULEPOS;
 
     /* Allow packetno == -1 to indicate oggz should fill it in; otherwise:
      * if op is the first packet in the stream, initialize the stream's
-     * packetno to the given one, otherwise check it for strictness */
+     * packetno to the given one, otherwise check it for strictness.
+     * For stream suffixes, believe the packetno value */
     if (op->packetno != -1) {
-      if (b_o_s) {
+      if (b_o_s || suffix) {
 	stream->packetno = op->packetno;
       } else if (op->packetno <= stream->packetno) {
 	return OGGZ_ERR_BAD_PACKETNO;
@@ -534,35 +539,35 @@ oggz_page_writeout (OGGZ * oggz, long n)
   return h + b;
 }
 
-static oggz_writer_packet_t *
-oggz_dequeue_packet (OGGZ * oggz)
+static int
+oggz_dequeue_packet (OGGZ * oggz, oggz_writer_packet_t ** next_zpacket)
 {
   OggzWriter * writer = &oggz->x.writer;
-  oggz_writer_packet_t * next_zpacket;
+  int ret = 0;
 
   if (writer->next_zpacket != NULL) {
 #ifdef DEBUG
     printf ("oggz_dequeue_packet: queue EMPTY\n");
 #endif
-    next_zpacket = writer->next_zpacket;
+    *next_zpacket = writer->next_zpacket;
     writer->next_zpacket = NULL;
   } else {
-    next_zpacket = oggz_vector_pop (writer->packet_queue);
+    *next_zpacket = oggz_vector_pop (writer->packet_queue);
 
 #ifdef DEBUG
     printf ("oggz_dequeue_packet: dequeued packet, queue size %d\n",
 	    oggz_vector_size (writer->packet_queue));
 #endif
 
-    if (next_zpacket == NULL) {
+    if (*next_zpacket == NULL) {
       if (writer->hungry) {
-	writer->hungry (oggz, 1, writer->hungry_user_data);
-	next_zpacket = oggz_vector_pop (writer->packet_queue);
+	ret = writer->hungry (oggz, 1, writer->hungry_user_data);
+	*next_zpacket = oggz_vector_pop (writer->packet_queue);
       }
     }
   }
 
-  return next_zpacket;
+  return ret;
 }
 
 static long
@@ -570,7 +575,7 @@ oggz_writer_make_packet (OGGZ * oggz)
 {
   OggzWriter * writer = &oggz->x.writer;
   oggz_writer_packet_t * zpacket, * next_zpacket;
-  int cb_ret = 0, ret = 0;
+  int cb_ret = 0;
 
 #ifdef DEBUG
   printf ("oggz_writer_make_packet: IN\n");
@@ -591,12 +596,12 @@ oggz_writer_make_packet (OGGZ * oggz)
 
   if (cb_ret == 0) {
     /* dequeue and init the next packet */
-    if ((next_zpacket = oggz_dequeue_packet (oggz)) == NULL) {
+    cb_ret = oggz_dequeue_packet (oggz, &next_zpacket);
+    if (next_zpacket == NULL) {
 #ifdef DEBUG
       printf ("oggz_writer_make_packet: packet queue empty\n");
 #endif
       /*writer->eos = 1;*/
-      ret = 0;
     } else {
       if ((writer->current_stream != NULL) &&
 	  (next_zpacket->flush & OGGZ_FLUSH_BEFORE)) {
@@ -610,13 +615,12 @@ oggz_writer_make_packet (OGGZ * oggz)
       } else {
 	oggz_packet_init (oggz, next_zpacket);
       }
-      ret = 1;
     }
-  } else {
-    ret = 0;
   }
 
-  return ret;
+  if (cb_ret == 0 && next_zpacket == NULL) return OGGZ_WRITE_EMPTY;
+
+  return cb_ret;
 }
 
 long
@@ -624,7 +628,7 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
 {
   OggzWriter * writer = &oggz->x.writer;
   long bytes, bytes_written = 1, remaining = n, nwritten = 0;
-  int active = 1;
+  int active = 1, cb_ret = 0;
 
   if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
 
@@ -635,11 +639,16 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
   if (writer->writing) return OGGZ_ERR_RECURSIVE_WRITE;
   writer->writing = 1;
 
+  if ((cb_ret = oggz->cb_next) != OGGZ_CONTINUE) {
+    oggz->cb_next = 0;
+    return oggz_map_return_value_to_error (cb_ret);
+  }
+
   while (active && remaining > 0) {
     bytes = MIN (remaining, 1024);
 
     while (writer->state == OGGZ_MAKING_PACKETS) {
-      if (!oggz_writer_make_packet (oggz)) {
+      if ((cb_ret = oggz_writer_make_packet (oggz)) != OGGZ_CONTINUE) {
 	active = 0;
 	break;
       }
@@ -653,7 +662,7 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
 
       if (bytes_written == -1) {
 	active = 0;
-	return OGGZ_ERR_SYSTEM; /* XXX: catch next */
+        cb_ret = OGGZ_ERR_SYSTEM; /* XXX: catch next */
       } else if (bytes_written == 0) {
 	if (!oggz_page_init (oggz)) {
 	  writer->state = OGGZ_MAKING_PACKETS;
@@ -669,6 +678,13 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
 
   writer->writing = 0;
 
+  if (nwritten == 0) {
+    if (cb_ret == OGGZ_WRITE_EMPTY) cb_ret = 0;
+    return oggz_map_return_value_to_error (cb_ret);
+  } else {
+    oggz->cb_next = cb_ret;
+  }
+
   return nwritten;
 }
 
@@ -677,7 +693,7 @@ oggz_write (OGGZ * oggz, long n)
 {
   OggzWriter * writer = &oggz->x.writer;
   long bytes, bytes_written = 1, remaining = n, nwritten = 0;
-  int active = 1;
+  int active = 1, cb_ret = 0;
 
   if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
 
@@ -692,6 +708,11 @@ oggz_write (OGGZ * oggz, long n)
   printf ("oggz_write: IN\n");
 #endif
 
+  if ((cb_ret = oggz->cb_next) != OGGZ_CONTINUE) {
+    oggz->cb_next = 0;
+    return oggz_map_return_value_to_error (cb_ret);
+  }
+
   while (active && remaining > 0) {
     bytes = MIN (remaining, 1024);
 
@@ -701,7 +722,7 @@ oggz_write (OGGZ * oggz, long n)
 #endif
 
     while (writer->state == OGGZ_MAKING_PACKETS) {
-      if (!oggz_writer_make_packet (oggz)) {
+      if ((cb_ret = oggz_writer_make_packet (oggz)) != OGGZ_CONTINUE) {
 	active = 0;
 #ifdef DEBUG
 	printf ("oggz_write: no packets\n");
@@ -739,6 +760,13 @@ oggz_write (OGGZ * oggz, long n)
 
   writer->writing = 0;
 
+  if (nwritten == 0) {
+    if (cb_ret == OGGZ_WRITE_EMPTY) cb_ret = 0;
+    return oggz_map_return_value_to_error (cb_ret);
+  } else {
+    oggz->cb_next = cb_ret;
+  }
+
   return nwritten;
 }
 
