@@ -59,33 +59,61 @@ typedef struct {
 } error_text;
 
 static error_text errors[] = {
-  {-5, "Multiple bos packets"},
-  {-6, "Multiple eos packets"},
   {-20, "Packet belongs to unknown serialno"},
   {-24, "Granulepos decreasing within track"},
+  {-5, "Multiple bos packets"},
+  {-6, "Multiple eos packets"},
   {0, NULL}
 };
 
+static int max_errors = MAX_ERRORS;
 static int multifile = 0;
 static char * current_filename = NULL;
 static timestamp_t current_timestamp = 0;
 static int exit_status = 0;
 static int nr_errors = 0;
+static int prefix = 0, suffix = 0;
 
+static void
+list_errors (void)
+{
+  int i = 0;
+
+  printf ("  File contains no Ogg packets\n");
+  printf ("  Packets out of order\n");
+  for (i = 0; errors[i].error; i++) {
+    printf ("  %s\n", errors[i].description);
+  }
+  printf ("  eos marked but no bos\n");
+  printf ("  Missing eos packets\n");
+  printf ("  Granulepos on page with no completed packets\n");
+  printf ("  Theora video bos page after audio bos page\n");
+}
 static void
 usage (char * progname)
 {
-  int i = 0;
 
   printf ("Usage: %s [options] filename ...\n", progname);
   printf ("Validate the Ogg framing of one or more files\n");
   printf ("\n%s detects the following errors in Ogg framing:\n", progname);
-  printf ("  Packet out of order\n");
-  for (i = 0; errors[i].error; i++) {
-    printf ("  %s\n", errors[i].description);
-  }
+
+  list_errors ();
+
+  printf ("\nError reporting options\n");
+  printf ("  -M num, --max-errors num\n");
+  printf ("                         Exit after the specified number of errors.\n");
+  printf ("                         A value of 0 specifies no maximum. Default: %d\n", MAX_ERRORS);
+  printf ("  -p, --prefix           Treat input as the prefix of a stream; suppress\n");
+  printf ("                         warnings about missing end-of-stream markers\n");
+  printf ("  -s, --suffix           Treat input as the suffix of a stream; suppress\n");
+  printf ("                         warnings about missing beginning-of-stream markers\n");
+  printf ("                         on the first chain\n");
+  printf ("  -P, --partial          Treat input as a the middle portion of a stream;\n");
+  printf ("                         equivalent to both --prefix and --suffix\n");
+
   printf ("\nMiscellaneous options\n");
   printf ("  -h, --help             Display this help and exit\n");
+  printf ("  -E, --help-errors      List known types of error and exit\n");
   printf ("  -v, --version          Output version information and exit\n");
   printf ("\n");
   printf ("Exit status is 0 if all input files are valid, 1 otherwise.\n\n");
@@ -102,7 +130,7 @@ log_error (void)
   exit_status = 1;
 
   nr_errors++;
-  if (nr_errors > MAX_ERRORS)
+  if (max_errors && nr_errors > max_errors)
     return OGGZ_STOP_ERR;
 
   return OGGZ_STOP_OK;
@@ -146,14 +174,17 @@ read_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
 
   if (ogg_page_bos ((ogg_page *)og)) {
     content_type = ot_page_identify (og, NULL);
-    if (!strcmp (content_type, "Theora")) {
-      ovdata->theora_count++;
-      if (ovdata->audio_count > 0) {
-        log_error ();
-        fprintf (stderr, "serialno %010ld: Theora video bos page after audio bos page\n", serialno);
+
+    if (content_type) {
+      if (!strcmp (content_type, "Theora")) {
+	ovdata->theora_count++;
+	if (ovdata->audio_count > 0) {
+	  log_error ();
+	  fprintf (stderr, "serialno %010ld: Theora video bos page after audio bos page\n", serialno);
+	}
+      } else if (!strcmp (content_type, "Vorbis") || !strcmp (content_type, "Speex")) {
+	ovdata->audio_count++;
       }
-    } else if (!strcmp (content_type, "Vorbis") || !strcmp (content_type, "Speex")) {
-      ovdata->audio_count++;
     }
   }
 
@@ -168,9 +199,15 @@ read_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
 static void
 ovdata_init (OVData * ovdata)
 {
+  int flags;
+
   current_timestamp = 0;
 
-  if ((ovdata->writer = oggz_new (OGGZ_WRITE|OGGZ_AUTO)) == NULL) {
+  flags = OGGZ_WRITE|OGGZ_AUTO;
+  if (prefix) flags |= OGGZ_PREFIX;
+  if (suffix) flags |= OGGZ_SUFFIX;
+
+  if ((ovdata->writer = oggz_new (flags)) == NULL) {
     fprintf (stderr, "oggz-validate: unable to create new writer\n");
     exit (1);
   }
@@ -188,7 +225,7 @@ ovdata_clear (OVData * ovdata)
 
   oggz_close (ovdata->writer);
 
-  if (nr_errors <= MAX_ERRORS) {
+  if (!prefix && (max_errors == 0 || nr_errors <= max_errors)) {
     nr_missing_eos = oggz_table_size (ovdata->missing_eos);
     for (i = 0; i < nr_missing_eos; i++) {
       log_error ();
@@ -212,12 +249,12 @@ read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
     oggz_table_insert (ovdata->missing_eos, serialno, (void *)0x1);
   }
 
-  if (oggz_table_lookup (ovdata->missing_eos, serialno) == NULL) {
+  if (!suffix && oggz_table_lookup (ovdata->missing_eos, serialno) == NULL) {
     ret = log_error ();
     fprintf (stderr, "serialno %010ld: missing *** bos\n", serialno);
   }
 
-  if (op->e_o_s) {
+  if (!suffix && op->e_o_s) {
     if (oggz_table_remove (ovdata->missing_eos, serialno) == -1) {
       ret = log_error ();
       fprintf (stderr, "serialno %010ld: *** eos marked but no bos\n",
@@ -269,6 +306,7 @@ read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   if (op->e_o_s && oggz_table_size (ovdata->missing_eos) == 0) {
     ovdata_clear (ovdata);
     ovdata_init (ovdata);
+    suffix = 0;
   }
 
   return ret;
@@ -289,9 +327,14 @@ validate (char * filename)
 
   /*printf ("oggz-validate: %s\n", filename);*/
 
-  if ((reader = oggz_open (filename, OGGZ_READ|OGGZ_AUTO)) == NULL) {
+  if (!strncmp (filename, "-", 2)) {
+    if ((reader = oggz_open_stdio (stdin, OGGZ_READ|OGGZ_AUTO)) == NULL) {
+      fprintf (stderr, "oggz-validate: unable to open stdin\n");
+      return -1;
+    }
+  } else if ((reader = oggz_open (filename, OGGZ_READ|OGGZ_AUTO)) == NULL) {
     fprintf (stderr, "oggz-validate: unable to open file %s\n", filename);
-    exit (1);
+    return -1;
   }
 
   ovdata_init (&ovdata);
@@ -300,7 +343,7 @@ validate (char * filename)
   oggz_set_read_page (reader, -1, read_page, &ovdata);
 
   while (active && (n = oggz_read (reader, 1024)) != 0) {
-    if (nr_errors > MAX_ERRORS) {
+    if (max_errors && nr_errors > max_errors) {
       fprintf (stderr,
 	       "oggz-validate: maximum error count reached, bailing out ...\n");
       active = 0;
@@ -327,6 +370,11 @@ main (int argc, char ** argv)
   int show_version = 0;
   int show_help = 0;
 
+  /* Cache the --prefix, --suffix options and reset before validating
+   * each input file */
+  int opt_prefix = 0;
+  int opt_suffix = 0;
+
   char * progname;
   char * filename;
   int i = 1;
@@ -341,11 +389,16 @@ main (int argc, char ** argv)
   }
 
   while (1) {
-    char * optstring = "hv";
+    char * optstring = "M:psPhvE";
 
 #ifdef HAVE_GETOPT_LONG
     static struct option long_options[] = {
+      {"max-errors", required_argument, 0, 'M'},
+      {"prefix", no_argument, 0, 'p'},
+      {"suffix", no_argument, 0, 's'},
+      {"partial", no_argument, 0, 'P'},
       {"help", no_argument, 0, 'h'},
+      {"help-errors", no_argument, 0, 'E'},
       {"version", no_argument, 0, 'v'},
       {0,0,0,0}
     };
@@ -355,7 +408,6 @@ main (int argc, char ** argv)
     i = getopt (argc, argv, optstring);
 #endif
     if (i == -1) {
-      i = 1;
       break;
     }
     if (i == ':') {
@@ -364,12 +416,28 @@ main (int argc, char ** argv)
     }
 
     switch (i) {
+    case 'M': /* max-errors */
+      max_errors = atoi (optarg);
+      break;
+    case 'p': /* prefix */
+      opt_prefix = 1;
+      break;
+    case 's': /* suffix */
+      opt_suffix = 1;
+      break;
+    case 'P': /* partial */
+      opt_prefix = 1;
+      opt_suffix = 1;
+      break;
     case 'h': /* help */
       show_help = 1;
       break;
     case 'v': /* version */
       show_version = 1;
       break;
+    case 'E': /* list errors */
+      show_help = 2;
+      break;
     default:
       break;
     }
@@ -379,14 +447,21 @@ main (int argc, char ** argv)
     printf ("%s version " VERSION "\n", progname);
   }
 
-  if (show_help) {
+  if (show_help == 1) {
     usage (progname);
+  } else if (show_help == 2) {
+    list_errors ();
   }
 
   if (show_version || show_help) {
     goto exit_out;
   }
 
+  if (max_errors < 0) {
+    printf ("%s: Error: [-M num, --max-errors num] option must be non-negative\n", progname);
+    goto exit_err;
+  }
+
   if (optind >= argc) {
     usage (progname);
     goto exit_err;
@@ -394,9 +469,12 @@ main (int argc, char ** argv)
 
   if (argc-i > 2) multifile = 1;
 
-  for (; i < argc; i++) {
+  for (i = optind; i < argc; i++) {
     filename = argv[i];
-    validate (filename);
+    prefix = opt_prefix;
+    suffix = opt_suffix;
+    if (validate (filename) == -1)
+      exit_status = 1;
   }
 
  exit_out:
