@@ -38,7 +38,9 @@
 #include <oggz/oggz.h>
 
 #ifdef WIN32                                                                   
-#include <fcntl.h>                                                             
+#include <fcntl.h>    
+#include <io.h>
+#define snprintf _snprintf
 #endif  
 
 static  ogg_uint32_t
@@ -174,6 +176,22 @@ ot_speex_info (unsigned char * data, long len)
   return buf;
 }
 
+static char *
+ot_oggpcm2_info (unsigned char * data, long len)
+{
+  char * buf;
+
+  if (len < 28) return NULL;
+
+  buf = malloc (60);
+
+  snprintf (buf, 60,
+	    "\tAudio-Samplerate: %d Hz\n\tAudio-Channels: %d\n",
+	    INT32_BE_AT(&data[16]), (int)data[21]);
+
+  return buf;
+}
+
 static char *
 ot_skeleton_info (unsigned char * data, long len)
 {
@@ -195,6 +213,7 @@ static const OTCodecIdent codec_ident[] = {
   {"\200theora", 7, "Theora", ot_theora_info},
   {"\001vorbis", 7, "Vorbis", ot_vorbis_info},
   {"Speex", 5, "Speex", ot_speex_info},
+  {"PCM     ", 8, "PCM", ot_oggpcm2_info},
   {"CMML\0\0\0\0", 8, "CMML", NULL},
   {"Annodex", 8, "Annodex", NULL},
   {"fishead", 8, "Skeleton", ot_skeleton_info},
@@ -293,12 +312,7 @@ ot_fprint_time (FILE * stream, double seconds)
   min = (int) ((seconds - ((double)hrs * 3600.0)) / 60.0);
   sec = seconds - ((double)hrs * 3600.0)- ((double)min * 60.0);
 
-  /* XXX: %02.3f workaround */
-  if (sec < 10.0) {
-    return fprintf (stream, "%s%02d:%02d:0%2.3f", sign, hrs, min, sec);
-  } else {
-    return fprintf (stream, "%s%02d:%02d:%02.3f", sign, hrs, min, sec);
-  }
+  return fprintf (stream, "%s%02d:%02d:%06.3f", sign, hrs, min, sec);
 }
 
 void
