@@ -51,6 +51,10 @@
 
 /*#define DEBUG*/
 
+#ifdef WIN32                                                                   
+#define strcasecmp _stricmp
+#endif  
+
 #undef MIN
 #define MIN(a,b) (((a)<(b))?(a):(b))
 
@@ -308,11 +312,13 @@ filter_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
 
   if (ogg_page_bos ((ogg_page *)og)) {
     ident = ot_page_identify (og, NULL);
-    n = oggz_table_size (oddata->content_types_table);
-    for (i = 0; i < n; i++) {
-      char * c = oggz_table_nth (oddata->content_types_table, i, NULL);
-      if (strcasecmp (c, ident) == 0) {
-	oggz_set_read_callback (oggz, serialno, oddata->read_packet, oddata);
+    if (ident) {
+      n = oggz_table_size (oddata->content_types_table);
+      for (i = 0; i < n; i++) {
+	char * c = oggz_table_nth (oddata->content_types_table, i, NULL);
+	if (strcasecmp (c, ident) == 0) {
+	  oggz_set_read_callback (oggz, serialno, oddata->read_packet, oddata);
+	}
       }
     }
   }
@@ -320,19 +326,15 @@ filter_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   return 0;
 }
 
-static int
-ignore_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
-{
-  return -1;
-}
-
 static int
 read_new_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 {
-  oggz_set_read_callback (oggz, serialno, ignore_packet, NULL);
-  read_packet (oggz, op, serialno, user_data);
-
-  return -1;
+  if (op->b_o_s) {
+    read_packet (oggz, op, serialno, user_data);
+    return OGGZ_CONTINUE;
+  } else {
+    return OGGZ_STOP_OK;
+  }
 }
 
 static void
@@ -657,7 +659,8 @@ main (int argc, char ** argv)
       }
     }
 
-    while ((n = oggz_read (oggz, 1024)) > 0);
+    oggz_run_set_blocksize (oggz, 1024*1024);
+    oggz_run (oggz);
 
     oggz_close (oggz);
   }
